local frame = CreateFrame("Frame",nil,UIParent)
local RESPONSEPREFIX = "REQUEUE_RESPONSE"
local REQUESTPREFIX = "REQUEUE_REQUEST"

local inQueue = {}
local notInQueue = {}

local function IsInQueue(bracket)
	bracket = tonumber(bracket)
	for i=1, MAX_BATTLEFIELD_QUEUES do
		local status = GetBattlefieldStatus(i)
		local teamSize,regMatch = select(6,GetBattlefieldStatus(i))
		if status == "queued" and teamSize == bracket then
		--if status == "queued" and teamSize == bracket and regMatch == 1 then
			return true
		end
	end
	return false
end

local _print = _G.print
local function print(s)
	_print(format("|cff99ff33Requeue|r: %s",s))
end

local TIMETOWAIT = 2
local _time = 0
local reported = {}
local function OnUpdate(self,elapsed)
	_time = _time + elapsed
	if _time > TIMETOWAIT then
		wipe(reported)
		-- Report people with queue
		for name in pairs(inQueue) do
			print("|cff00ff00"..name.." is in the queue")
			reported[name] = true
		end
		-- Report people without queue
		for name in pairs(notInQueue) do
			print("|cffff0000"..name.." is NOT in the queue|r")
			reported[name] = true
		end
		-- Report people without the mod
		for i=1,GetNumPartyMembers() do
			local uid = "party"..i
			if not UnitIsUnit(uid,"player") and not reported[UnitName(uid)] then
				print("|cff808080"..UnitName(uid).."'s queue status is unknown|r")
			end
		end
		self:SetScript("OnUpdate",nil)
		_time = 0
	end
end

local function RequestQueues()
	print("Requesting queue statuses from party members...")
	if not frame:GetScript("OnUpdate") then frame:SetScript("OnUpdate",OnUpdate) end
	wipe(inQueue); wipe(notInQueue)
	for i=1, MAX_BATTLEFIELD_QUEUES do
		local teamSize,regMatch = select(6,GetBattlefieldStatus(i))
		if teamSize > 0  then
		--if teamSize > 0 and regMatch == 1 then
			SendAddonMessage(REQUESTPREFIX,tostring(teamSize),"PARTY")
			break
		end
	end
end

local function OnEvent(self,event,prefix,msg,dist,sender)
	if event == "UPDATE_BATTLEFIELD_STATUS" then
		local status
		for i=1, MAX_BATTLEFIELD_QUEUES do
			status = GetBattlefieldStatus(i)
			local teamSize,regMatch = select(6,GetBattlefieldStatus(i))
			--if status == "queued" and teamsize > 0 and regMatch == 1 then break end
			if status == "queued" and teamSize > 0 then break end
		end		
		if select(2,IsInInstance()) ~= "arena" and status == "queued" and GetNumPartyMembers() > 0 and IsPartyLeader() then
			RequestQueues()
		end
	elseif event == "CHAT_MSG_ADDON" then
		if sender == UnitName("player") then return end
		if prefix == RESPONSEPREFIX and dist == "WHISPER" then
			if msg == "YES" then
				inQueue[sender] = true
			elseif msg == "NO" then
				notInQueue[sender] = true
			end
		elseif prefix == REQUESTPREFIX and dist == "PARTY" then
			SendAddonMessage(RESPONSEPREFIX,IsInQueue(msg) and "YES" or "NO","WHISPER",sender)
		end
	end
end

frame:SetScript("OnEvent",OnEvent)
frame:RegisterEvent("CHAT_MSG_ADDON")
frame:RegisterEvent("UPDATE_BATTLEFIELD_STATUS")
